@@ -45,6 +45,7 @@ from guake.globals import ALIGN_LEFT
 from guake.globals import ALIGN_RIGHT
 from guake.globals import ALIGN_TOP
 from guake.globals import ALWAYS_ON_PRIMARY
+from guake.globals import ENGINES
 from guake.globals import MAX_TRANSPARENCY
 from guake.globals import NAME
 from guake.globals import QUICK_OPEN_MATCHERS
@@ -103,6 +104,7 @@ HOTKEYS = [
         "keys": [
             {"key": "new-tab", "label": _("New tab")},
             {"key": "new-tab-home", "label": _("New tab in home directory")},
+            {"key": "new-tab-cwd", "label": _("New tab in current directory")},
             {"key": "close-tab", "label": _("Close tab")},
             {"key": "rename-current-tab", "label": _("Rename current tab")},
         ],
@@ -141,7 +143,9 @@ HOTKEYS = [
         "key": "nav",
         "keys": [
             {"key": "previous-tab", "label": _("Go to previous tab")},
+            {"key": "previous-tab-alt", "label": _("Go to previous tab (alternative)")},
             {"key": "next-tab", "label": _("Go to next tab")},
+            {"key": "next-tab-alt", "label": _("Go to next tab (alternative)")},
             {"key": "move-tab-left", "label": _("Move current tab left")},
             {"key": "move-tab-right", "label": _("Move current tab right")},
             {"key": "switch-tab1", "label": _("Go to first tab")},
@@ -177,6 +181,7 @@ HOTKEYS = [
         "keys": [
             {"key": "clipboard-copy", "label": _("Copy text to clipboard")},
             {"key": "clipboard-paste", "label": _("Paste text from clipboard")},
+            {"key": "select-all", "label": _("Select all")},
         ],
     },
     {
@@ -184,6 +189,10 @@ HOTKEYS = [
         "key": "extra",
         "keys": [
             {"key": "search-on-web", "label": _("Search selected text on web")},
+            {
+                "key": "open-link-under-terminal-cursor",
+                "label": _("Open URL under terminal cursor"),
+            },
         ],
     },
 ]
@@ -245,6 +254,10 @@ class PrefsCallbacks:
         """Changes the activity of save-tabs-when-changed in dconf"""
         self.settings.general.set_boolean("save-tabs-when-changed", chk.get_active())
 
+    def on_load_guake_yml_toggled(self, chk):
+        """Changes the activity of load-guake-yml"""
+        self.settings.general.set_boolean("load-guake-yml", chk.get_active())
+
     def on_default_shell_changed(self, combo):
         """Changes the activity of default_shell in dconf"""
         citer = combo.get_active_iter()
@@ -286,6 +299,24 @@ class PrefsCallbacks:
         """Set the `prompt_on_close_tab' property in dconf"""
         self.settings.general.set_int("prompt-on-close-tab", combo.get_active())
 
+    def on_search_engine_changed(self, combo):
+        """
+        Sets the 'search-engine' value in dnonf.
+        Also controls the editability of 'custom_search' input
+        """
+        custom_search = self.prefDlg.get_widget("custom_search")
+        # if 'Custom' is selected make the search engine input editable
+        if combo.get_active() not in ENGINES:
+            custom_search.set_sensitive(True)
+        else:
+            # make read-only
+            custom_search.set_sensitive(False)
+        self.settings.general.set_int("search-engine", combo.get_active())
+
+    def on_custom_search_changed(self, edt):
+        """Sets the 'custom-search-engine' property in dconf"""
+        self.settings.general.set_string("custom-search-engine", edt.get_text())
+
     def on_gtk_theme_name_changed(self, combo):
         """Set the `gtk_theme_name' property in dconf"""
         citer = combo.get_active_iter()
@@ -335,6 +366,10 @@ class PrefsCallbacks:
         """Changes the activity of window_losefocus in dconf"""
         self.settings.general.set_boolean("window-losefocus", chk.get_active())
 
+    def on_lazy_lose_focus_toggled(self, chk):
+        """Changes the activity of lazy_lose_focus in dconf"""
+        self.settings.general.set_boolean("lazy-losefocus", chk.get_active())
+
     def on_quick_open_command_line_changed(self, edt):
         self.settings.general.set_string("quick-open-command-line", edt.get_text())
 
@@ -488,6 +523,14 @@ class PrefsCallbacks:
         """Changes the value of bold_is_bright in dconf"""
         self.settings.styleFont.set_boolean("bold-is-bright", chk.get_active())
 
+    def on_cell_height_scale_value_changed(self, scale):
+        value = scale.get_value()
+        self.settings.styleFont.set_double("cell-height-scale", value)
+
+    def on_cell_width_scale_value_changed(self, scale):
+        value = scale.get_value()
+        self.settings.styleFont.set_double("cell-width-scale", value)
+
     def on_font_style_font_set(self, fbtn):
         """Changes the value of font_style in dconf"""
         self.settings.styleFont.set_string("style", fbtn.get_font_name())
@@ -545,6 +588,9 @@ class PrefsCallbacks:
     def toggle_show_tabbar_sensitivity(self, chk):
         self.prefDlg.toggle_show_tabbar_sensitivity(chk)
 
+    def toggle_hide_on_lose_focus_sensitivity(self, chk):
+        self.prefDlg.toggle_hide_on_lose_focus_sensitivity(chk)
+
     def toggle_quick_open_command_line_sensitivity(self, chk):
         self.prefDlg.toggle_quick_open_command_line_sensitivity(chk)
 
@@ -769,6 +815,12 @@ class PrefsDialog(SimpleGladeApp):
         self.get_widget("palette_16").set_sensitive(chk.get_active())
         self.get_widget("palette_17").set_sensitive(chk.get_active())
 
+    def toggle_hide_on_lose_focus_sensitivity(self, chk):
+        """If the user chooses to not hide Guake on focus loss, lazy hiding on focus
+        loss will do nothing.
+        """
+        self.get_widget("lazy_lose_focus").set_sensitive(chk.get_active())
+
     def toggle_show_tabbar_sensitivity(self, chk):
         """If the user chooses to not show the tab bar, it means that they
         cannot see the tab bar regardless of what other tab bar options say.
@@ -985,6 +1037,10 @@ class PrefsDialog(SimpleGladeApp):
         value = self.settings.general.get_boolean("save-tabs-when-changed")
         self.get_widget("save-tabs-when-changed").set_active(value)
 
+        # save tabs when changed
+        value = self.settings.general.get_boolean("load-guake-yml")
+        self.get_widget("load-guake-yml").set_active(value)
+
         # login shell
         value = self.settings.general.get_boolean("use-login-shell")
         self.get_widget("use_login_shell").set_active(value)
@@ -1010,6 +1066,18 @@ class PrefsDialog(SimpleGladeApp):
         self.get_widget("prompt_on_close_tab").set_active(value)
         self.get_widget("prompt_on_quit").set_sensitive(value != 2)
 
+        # search engine
+        value = self.settings.general.get_int("search-engine")
+        custom_search = self.get_widget("custom_search")
+        custom_search.set_text(self.settings.general.get_string("custom-search-engine"))
+        self.get_widget("search_engine_select").set_active(value)
+        # if 'Custom' is selected make the search engine input editable
+        if value not in ENGINES:
+            # make read-only
+            custom_search.set_sensitive(True)
+        else:
+            custom_search.set_sensitive(False)
+
         # use system theme
         value = self.settings.general.get_boolean("gtk-use-system-default-theme")
         self.get_widget("gtk_use_system_default_theme").set_active(value)
@@ -1042,6 +1110,10 @@ class PrefsDialog(SimpleGladeApp):
         value = self.settings.general.get_boolean("window-losefocus")
         self.get_widget("window_losefocus").set_active(value)
 
+        # lazy lose focus
+        value = self.settings.general.get_boolean("lazy-losefocus")
+        self.get_widget("lazy_lose_focus").set_active(value)
+
         # use VTE titles
         value = self.settings.general.get_boolean("use-vte-titles")
         self.get_widget("use_vte_titles").set_active(value)
@@ -1180,6 +1252,14 @@ class PrefsDialog(SimpleGladeApp):
         value = self.settings.styleFont.get_boolean("bold-is-bright")
         self.get_widget("bold_is_bright").set_active(value)
 
+        # cell height scale
+        value = self.settings.styleFont.get_double("cell-height-scale")
+        self.get_widget("cell_height_scale_adjustment").set_value(value)
+
+        # cell width scale
+        value = self.settings.styleFont.get_double("cell-width-scale")
+        self.get_widget("cell_width_scale_adjustment").set_value(value)
+
         # background image file
         filename = self.settings.general.get_string("background-image-file")
         if os.path.exists(filename):
